%%
%% Matlab Test Code - Side-Channel Statistical Toolbox
%% Copyright  Franois-Xavier Standaert
%% e-mail: <fstandae@uclouvain.be>
%%
%% This program is a free software: you can redistribute it 
%% and/or modify it under the terms of the GNU General Public 
%% License as published by the Free Software Foundation.
%%
%% It is distributed without any warranty of correctness nor
%% fintess for any particular purpose. See the GNU General 
%% Public License for more details.
%% 
%% <http://www.gnu.org/licenses/>
%%
%% Description: computation of the normalized aggregated MI 
%% NAMI(c) for a given k, with L=HW(Y)+N and N a Gaussian noise, 
%% for different time complexities c, via numerical integration 
%% 
%% Version 1 - January 2016

clear
warning off all
%--------------------------------------------------------------------------
% Main parameters
%--------------------------------------------------------------------------
n=8; % bitsize of Y
sigma=sqrt(20); % noise level
k=0;  % target key value
b=0; % 1 for strict bound, 0 for heuristic 
%--------------------------------------------------------------------------

% vector of Hamming weights 
for i=1:2^n
    W(i)=hamming(i-1,n);
end

% vector of best Hamming weights given a Hamming weight h
for h=0:n
    BHC(h+1,:)=best_Wy_candidates(h,n);
end

h = waitbar(0,'Please wait...');
for i=1:2^n % for 2^n different time complexities
    waitbar(i/2^n);
    HYL_int(i)=0; % H[Y|L] computed via integration
    for Wy=0:n % for all HW(y)   
        AWy=BHC(Wy+1,1:i); % models for the aggregated target intermediate values
        F = @(l) (normpdf(l,Wy,sigma)*log2(sum(normpdf(l,AWy,sigma))/sum(normpdf(l,W,sigma))));
        HYL_int(i)=HYL_int(i)-nchoosek(n,Wy)/(2^n)*quadv(F,Wy-5*sigma,Wy+5*sigma);
    end
    if b==1 % normalized aggregated MI (strict bound)
        if (-log2(i/2^n)-HYL_int(i))/(-log2(i/2^n))*n<1
            NAMI(i)=(-log2(i/2^n)-HYL_int(i))/(-log2(i/2^n))*n;
        else
            NAMI(i)=1;
        end
    else % normalized aggregated MI (heuristic)
        if (-log2(i/2^n)-HYL_int(i))/(-log2(i/2^n))*sqrt(n)<1
            NAMI(i)=(-log2(i/2^n)-HYL_int(i))/(-log2(i/2^n))*sqrt(n);
        else
            NAMI(i)=1;
        end
    end
end
NAMI(2^n)=1;
save NAMI NAMI;
SNR=(n/4)/(sigma^2);
save SNR SNR;
save b b;
save n n;
close(h);







